package modele

import views.ViewsGame

class DashBoard(private val width: Int,private val height: Int,private val view: ViewsGame) {
    private val gridOfGame = Array(this.height) { IntArray(this.width) }

    // Our getter
    fun getWidth(): Int = this.width
    fun getHeight(): Int = this.height

    // To set something to occupied

    fun toOccupied(col: Int, row: Int, value: Int) {
        gridOfGame[row][col] = value
    }

    // To check if an position is occupied
    fun isOccupied(col: Int, row: Int): Boolean = gridOfGame[row][col] != 0

    fun isLineFull(row: Int): Boolean {
        for (col in 0 until this.width) {
            if (gridOfGame[row][col] == 0) {
                return false
            }
        }
        return true
    }

    fun clearLine(row: Int) {
        for (col in 0 until this.width) {
            gridOfGame[row][col] = 0
        }
    }

    fun shiftDown(rowToBegin: Int) {
        for (row in (height - 1)..rowToBegin) {
            for (col in 0 until this.width) {
                gridOfGame[row][col] = gridOfGame[row - rowToBegin][col]
            }
        }
        for (row in 0 until rowToBegin) {
            for (col in 0 until this.width) {
                gridOfGame[row][col] = 0
            }
        }
    }

    //To check each grid line and remove if a line is full. Uses clearLine(), isLineFull() and shiftDown()
    fun clearLines() {
        var nbRowCleared: Int = 0
        for (row in 0 until this.height) {
            if (isLineFull(row)) {
                clearLine(row)
                ++nbRowCleared
            }
        }
        if (nbRowCleared != 0) {
            shiftDown(nbRowCleared)
        }
    }

    fun addShape(shape: Shape): Boolean {
        when (shape.typeShape.type) {
            EnumTypeShape.IShape -> {  // Shape I (la ligne)
                // Mettre ICI des verification ou un try pour les limite du tableau

                for (line in 0..3) { // On vérifie que l'espace est disponible
                    for (column in 0..3) {
                        if ((isOccupied(
                                shape.position.getX() + column,
                                shape.position.getY() + line
                            )) and (shape.typeShape.showShape[line][column] == 1)
                        ) {
                            return false
                        }
                    }
                }
                writeShape(shape)
            }
            else -> {
                println("Type de piece incomprehensible !!")
                return false
            }
        }
        return false
    }


    // ATTENTION ELLE BUG PAS DE COLLISION
    fun moveRight(shape:Shape):Position{
        //Check les limite du tableau à droite !!!! NE MARCHE PAS ATTENTION !!!
        var limite = width-shape.position.getX()
        if (limite >= 4 ){
            limite = 4
        }

        for ( line in 0..3){  // Check si le décalage est possible
            for ( column in 0..3){
                if ( column != 3) {
                    if ((shape.typeShape.showShape[line][column] == 1)
                        and (shape.typeShape.showShape[line][column + 1] == 0)
                        and (gridOfGame[shape.position.getX() + line][shape.position.getY() + column + 1] == 1)
                    ) {
                        return shape.position
                    }
                }else{
                    if ((shape.typeShape.showShape[line][column] == 1)
                        and (shape.typeShape.showShape[line][column] == 0)
                        and (gridOfGame[shape.position.getX() + line][shape.position.getY() + column + 1] == 1)
                    ) {
                        return shape.position
                    }
                }
            }
        }
        deleteShape(shape)
        shape.position.addX()
        writeShape(shape)
        return shape.position
    }

    // Delete a shape in gridOfGame
    private fun deleteShape(shape: Shape){
        for (line in 0..3) {
            for (column in 0..3) {
                if (shape.typeShape.showShape[line][column] == 1) {
                    gridOfGame[shape.position.getX() + line][shape.position.getY() + column] = 0
                }
            }
        }
    }

    // Write the shape in gridOfGame
    private fun writeShape(shape: Shape){
        for (line in 0..3) {
            for (column in 0..3) {
                if (shape.typeShape.showShape[line][column] == 1) {
                    gridOfGame[shape.position.getY() + line][shape.position.getX() + column] =
                        shape.typeShape.couleur
                }
            }
        }
        updateViewGame()
    }

    // Update the view
    fun updateViewGame(){
        println("-----------------")
        view.tableau = gridOfGame
    }
}